@@ -167,7 +167,7 @@ _xdg_mime_magic_read_to_newline (FILE *magic_file,
 
   while (TRUE)
     {
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
       if (c == EOF)
 	{
 	  *end_of_file = TRUE;
@@ -195,21 +195,21 @@ _xdg_mime_magic_read_a_number (FILE *magic_file,
 {
   /* LONG_MAX is about 20 characters on my system */
 #define MAX_NUMBER_SIZE 30
-  char number_string[MAX_NUMBER_SIZE];
+  char number_string[MAX_NUMBER_SIZE + 1];
   int pos = 0;
   int c;
   long retval = -1;
 
   while (TRUE)
     {
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
 
       if (c == EOF)
 	{
 	  *end_of_file = TRUE;
 	  break;
 	}
-      if (! isdigit ((char) c))
+      if (! isdigit (c))
 	{
 	  ungetc (c, magic_file);
 	  break;
@@ -246,7 +246,7 @@ _xdg_mime_magic_parse_header (FILE *magic_file, XdgMimeMagicMatch *match)
   assert (magic_file != NULL);
   assert (match != NULL);
 
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     return XDG_MIME_MAGIC_EOF;
   if (c != '[')
@@ -258,7 +258,7 @@ _xdg_mime_magic_parse_header (FILE *magic_file, XdgMimeMagicMatch *match)
   if (match->priority == -1)
     return XDG_MIME_MAGIC_ERROR;
 
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     return XDG_MIME_MAGIC_EOF;
   if (c != ':')
@@ -291,7 +291,7 @@ _xdg_mime_magic_parse_error (FILE *magic_file)
 
   while (1)
     {
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
       if (c == EOF)
 	return XDG_MIME_MAGIC_EOF;
       if (c == '\n')
@@ -316,7 +316,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
   assert (magic_file != NULL);
 
   /* Sniff the buffer to make sure it's a valid line */
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     return XDG_MIME_MAGIC_EOF;
   else if (c == '[')
@@ -337,7 +337,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 	return XDG_MIME_MAGIC_EOF;
       if (indent == -1)
 	return XDG_MIME_MAGIC_ERROR;
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
       if (c == EOF)
 	return XDG_MIME_MAGIC_EOF;
     }
@@ -358,7 +358,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
       _xdg_mime_magic_matchlet_free (matchlet);
       return XDG_MIME_MAGIC_ERROR;
     }
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     {
       _xdg_mime_magic_matchlet_free (matchlet);
@@ -372,7 +372,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 
   /* Next two bytes determine how long the value is */
   matchlet->value_length = 0;
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     {
       _xdg_mime_magic_matchlet_free (matchlet);
@@ -381,7 +381,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
   matchlet->value_length = c & 0xFF;
   matchlet->value_length = matchlet->value_length << 8;
 
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == EOF)
     {
       _xdg_mime_magic_matchlet_free (matchlet);
@@ -407,7 +407,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 	return XDG_MIME_MAGIC_ERROR;
     }
 
-  c = fgetc (magic_file);
+  c = getc_unlocked (magic_file);
   if (c == '&')
     {
       matchlet->mask = malloc (matchlet->value_length);
@@ -426,7 +426,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 	  else
 	    return XDG_MIME_MAGIC_ERROR;
 	}
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
     }
 
   if (c == '~')
@@ -445,7 +445,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 	  _xdg_mime_magic_matchlet_free (matchlet);
 	  return XDG_MIME_MAGIC_ERROR;
 	}
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
     }
 
   if (c == '+')
@@ -461,7 +461,7 @@ _xdg_mime_magic_parse_magic_line (FILE              *magic_file,
 	  _xdg_mime_magic_matchlet_free (matchlet);
 	  return XDG_MIME_MAGIC_ERROR;
 	}
-      c = fgetc (magic_file);
+      c = getc_unlocked (magic_file);
     }
 
 
@@ -635,8 +635,10 @@ _xdg_mime_magic_new (void)
 void
 _xdg_mime_magic_free (XdgMimeMagic *mime_magic)
 {
-  if (mime_magic)
+  if (mime_magic) {
+    _xdg_mime_magic_match_free (mime_magic->match_list);
     free (mime_magic);
+  }
 }
 
 int
@@ -691,17 +693,16 @@ _xdg_mime_magic_matchlet_mirror (XdgMimeMagicMatchlet *matchlets)
 {
   XdgMimeMagicMatchlet *new_list;
   XdgMimeMagicMatchlet *tmp;
-  
-  if ((matchlets == NULL) || (matchlets->next == NULL)) 
-    {
-      return matchlets;
-    }
+
+  if ((matchlets == NULL) || (matchlets->next == NULL))
+    return matchlets;
 
   new_list = NULL;
   tmp = matchlets;
-  while (tmp != NULL) 
+  while (tmp != NULL)
     {
       XdgMimeMagicMatchlet *matchlet;
+
       matchlet = tmp;
       tmp = tmp->next;
       matchlet->next = new_list;
@@ -709,7 +710,7 @@ _xdg_mime_magic_matchlet_mirror (XdgMimeMagicMatchlet *matchlets)
     }
 
   return new_list;
-  
+
 }
 
 static void
@@ -733,7 +734,8 @@ _xdg_mime_magic_read_magic_file (XdgMimeMagic *mime_magic,
 	  break;
 	case XDG_MIME_MAGIC_MAGIC:
 	  state = _xdg_mime_magic_parse_magic_line (magic_file, match);
-	  if (state == XDG_MIME_MAGIC_SECTION) 
+	  if (state == XDG_MIME_MAGIC_SECTION ||
+	      (state == XDG_MIME_MAGIC_EOF && match->mime_type))
 	    {
 	      match->matchlet = _xdg_mime_magic_matchlet_mirror (match->matchlet);
 	      _xdg_mime_magic_insert_match (mime_magic, match);
